%PXMWRITE:  Write a PXM (pbm, pgm, ppm) file to disk.
%
%  PXMWRITE('filname',pxmformat,I)
%  writes a PBM file containing the bitmap image
%  in the matrix I. The pxmformat must be 'b'.  
%  The values of I must be either 1 or 0.
%
%  PXMWRITE('filname',pxmformat,maxgray,I)
%  writes the image in matrix I as a pbm file
%  when pxmformat == 'b'. It  writes I as a 
%  pgm format file if pxmformat == 'g'.
%  The values of "I" should be between 0 and 1.
%  The values of maxgray should be an integer
%  between 1 and 255.
%
%  PXMWRITE('filname',pxmformat,maxgray,I,CM)
%  writes the indexed image in matrix I and 
%  color map in CM  as a pgm format file
%  when pxmformat == 'g', and as a ppm format
%  file when pxmformat == 'p'.  
%  When pxmformat == 'g',  only second component 
%  of  CM is considered.
%  The values of "I" should between 0 and 1.
%  The values of maxgray should be an integer
%  between 1 and 255.
%
%  PXMWRITE(filename,pxmformat,maxgray,R,G,B)
%  writes the RGB image in the matrices R,G,B 
%  into a ppm format file. The pxmformat must 
%  be 'p'. and the values of R,G,B should between 
%  0 and 1.  The values of maxgray should be an 
%  integer between 1 and 255.
%
%  See also PXMREAD.

%  Written by Hui Cheng  			Jan 21, 1996 
%  email: hui@ecn.purdue.edu

function pxmread(arg1,arg2,arg3,arg4,arg5,arg6)

if (nargin<3)
	error('Not enough input arguments.') ;
end ;

if (isstr(arg1)~=1)
	error('Requires a string filename as an argument.') ;
end ;

if (isstr(arg2)~=1)
	error('Requires a string pxmformat as an argument.') ;
end ;

if (nargin > 3)
	if (arg3<1)|(arg3>255)
		error('Maxgrayvalues must be in [1,255].') ;
	end
end

if (nargin==3)
	if (arg2=='b')
		TempM = arg3 ;
		[nR,nC] = size(TempM) ;
	else
		error('Invalid pxmformat or pxmformat mismatch.') ;
	end
	if (find(TempM~=1 & TempM~=0)~=[])
		error('Invalid input.  Data should be 1 or 0.');
	end
end

if (nargin==4)
	if (arg2=='b')
		TempM = arg4 ;
		[nR,nC] = size(TempM) ;
		if (find(TempM~=1 & TempM~=0)~=[])
			error('Invalid input.  Data should be 1 or 0.');
		end
	elseif (arg2=='g')
		TempM = arg4 ;
		[nR,nC] = size(TempM) ;
		if (find(TempM>1 | TempM<0)~=[])
		    error('Invalid input. Data should be between 1 and 0.');
		end
	else
		error('Invalid pxmformat or pxmformat mismatch.') ;
	end
end

if (nargin==5)
	if  (arg2=='g')
		[nR,nC] = size(arg4) ;
		TempM = -1*ones(nR,nC) ;
		for(i=1:nC)
			TempM(:,i) = arg5(arg4(:,i),2) ;
		end
	elseif (arg2=='p')
		[nR,nC] = size(arg4) ;
		TempM = -1*ones(nR,nC*3) ;
		for(i=1:nC)
			TempM(:,3*i-2:3*i) = arg5(arg4(:,i),1:3) ;
		end
		TempM = TempM' ;
	else
		error('Invalid pxmformat or pxmformat mismatch.') ;
	end
	
	if (find(TempM>1 | TempM<0)~=[])
	 	error('Invalid input. Data should be between 1 and 0.');
	end
end
			
if (nargin==6)
	if (arg2=='p')
		if (size(arg4)~=size(arg5) | size(arg5)~=size(arg6))
			error('R,G,B should have the same size.') ;
		end
		
		[nR,nC] = size(arg4) ;
		TempM = -1*ones(nC*3,nR) ;
		Ind = 1:3:nC*nR*3 ;
		TempM(Ind) = arg4' ;
		TempM(Ind+1) = arg5' ;
		TempM(Ind+2) = arg6' ;
	else
		error('Invalid pxmformat or pxmformat mismatch.') ;
	end
	
	if (find(TempM>1 | TempM<0)~=[])
	 	error('Invalid input. Data should be between 1 and 0.');
	end
end
		
fid = fopen(arg1,'wb');
if (fid==-1)
	error(['Error opening ', arg1, ' for input.']) ;
end

if (arg2=='b')
	fprintf(fid,'P4\n%d\n%d\n',nC,nR);
	nC1 = round(nC/8 +.5) ;
	B1 = zeros(nC1*8,nR) ;
	B1(1:nC,:) = TempM' ;  % reshape a to a square matrix
	TempM = zeros(nC1,nR) ;
	C1 = zeros(nC1,nR) ;
	Index1 = 1:8:8*nC1*nR ;
	mask = [128,64,32,16,8,4,2,1] ;
	for (i1=0:7)
		C1 = B1(Index1+i1) ;
		Index2 = find(C1>0) ;
		TempM(Index2)=TempM(Index2)+mask(i1+1)*ones(size(Index2));
	end	
	fwrite(fid, round(TempM), 'integer*1');
	
elseif (arg2=='g')
	fprintf(fid,'P5\n%d\n%d\n%d\n',nC,nR,arg3);
	TempM = round(arg3*TempM)' ;
	fwrite(fid, TempM , 'integer*1');

elseif (arg2=='p')
	fprintf(fid,'P6\n%d\n%d\n%d\n',nC,nR,arg3);
	TempM = round(arg3*TempM) ;
	fwrite(fid, TempM , 'integer*1');

else 
	error('Invalid pxmformat') ;
end
